/* 
   (c) Copyright 2001-2002  convergence integrated media GmbH.
   (c) Copyright 2002-2005  convergence GmbH.

   All rights reserved.

   Written by Denis Oliver Kropp <dok@directfb.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>

#include "lite_internal.h"

#include "util.h"
#include "image.h"

D_DEBUG_DOMAIN(LiteImageDomain, "LiTE/Image", "LiteImage");

LiteImageTheme *liteDefaultImageTheme = NULL;

struct _LiteImage {
     LiteBox           box;
     LiteImageTheme   *theme;

     int               width, height;
     IDirectFBSurface *surface;
};


static DFBResult draw_image(LiteBox *box, const DFBRegion *region, DFBBoolean clear);
static DFBResult destroy_image(LiteBox *box);

DFBResult 
lite_new_image(LiteBox        *parent, 
               DFBRectangle   *rect,
               LiteImageTheme *theme,
               LiteImage     **ret_image)
{
     LiteImage *image = NULL;
     DFBResult  res;

     image = calloc(1, sizeof(LiteImage));

     image->box.parent = parent;
     image->theme = theme;

     image->box.rect.x = rect->x;
     image->box.rect.y = rect->y;
     image->box.rect.w = rect->w;
     image->box.rect.h = rect->h;

     res = lite_init_box(LITE_BOX(image));
     if (res != DFB_OK) {
          free(image);
          return res;
     }

     image->box.Draw    = draw_image;
     image->box.Destroy = destroy_image;
     
     *ret_image = image;

     return DFB_OK;
}

DFBResult 
lite_load_image(LiteImage *image, const char *filename)
{
     DFBResult res;

     LITE_NULL_PARAMETER_CHECK(image);
     LITE_NULL_PARAMETER_CHECK(filename);

     res = lite_util_load_image(filename, DSPF_UNKNOWN, &image->surface,
                               &image->width, &image->height, NULL);
     if(res != DFB_OK)
          return res;

     return lite_update_box(LITE_BOX(image), NULL);

}

/* internals */

static DFBResult 
destroy_image(LiteBox *box)
{
     LiteImage *image = LITE_IMAGE(box);

     D_ASSERT(box != NULL);

     if (!image)
          return DFB_FAILURE;

     if (image->surface)
          image->surface->Release(image->surface);

     return lite_destroy_box(box);
}

static DFBResult 
draw_image(LiteBox         *box, 
           const DFBRegion *region, 
           DFBBoolean       clear)
{
     LiteImage        *image   = LITE_IMAGE(box);
     IDirectFBSurface *surface = box->surface;

     D_ASSERT(box != NULL);

     surface->SetClip(surface, region);

     if (image->surface) {
          surface->SetBlittingFlags(surface, DSBLIT_BLEND_ALPHACHANNEL);
          surface->StretchBlit(surface, image->surface, NULL, NULL);
     }
     else if (clear)
          lite_clear_box(box, region);

     return DFB_OK; 
}
